/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * Voice.cpp
 *
 *  Created on: Aug 21, 2009
 *      Author: jwienke
 */

#include "Voice.h"

using namespace std;
using namespace xmltio;

namespace airobots {

TimeUnit iTU = MICROSECONDS;
AngleUnit iAU = RADIAN;

Voice::Voice() :
		angle(0), runtimeDifference(0), angleVariance(0), runtimeDifferenceVariance(
				0) {
}

Voice::~Voice() {
}

std::string Voice::getBaseTag() {
	return "VOICE";
}

std::string Voice::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<POSITION angle=\"\" angleVariance=\"\" runtimeDifferenceMicro=\"\" runtimeDifferenceVarianceMicro=\"\"/>"
					"</" + getBaseTag() + ">";
}

std::string Voice::getPerceptName() {
	return "voice";
}

float Voice::getAngle(AngleUnit angleUnit) const {
	return UnitConverter::convert(angle, iAU, angleUnit);
}

void Voice::setAngle(float angle, AngleUnit angleUnit) {
	this->angle = UnitConverter::convert(angle, angleUnit, iAU);
}

float Voice::getAngleVariance(AngleUnit angleUnit) const {
	return UnitConverter::convert(angleVariance, iAU, angleUnit);
}

void Voice::setAngleVariance(float angleVariance, AngleUnit angleUnit) {
	this->angleVariance = UnitConverter::convert(angleVariance, angleUnit, iAU);
}

long Voice::getRuntimeDifference(TimeUnit timeUnit) const {
	return UnitConverter::convert(runtimeDifference, iTU, timeUnit);
}

void Voice::setRuntimeDifference(long runtimeDifference, TimeUnit timeUnit) {
	this->runtimeDifference = UnitConverter::convert(runtimeDifference,
			timeUnit, iTU);
}

long Voice::getRuntimeDifferenceVariance(TimeUnit timeUnit) const {
	return UnitConverter::convert(runtimeDifferenceVariance, iTU, timeUnit);
}

void Voice::setRuntimeDifferenceVariance(long runtimeDifferenceVariance,
		TimeUnit timeUnit) {
	this->runtimeDifferenceVariance = UnitConverter::convert(
			runtimeDifferenceVariance, timeUnit, iTU);
}

bool Voice::operator==(const Voice &other) const {

	return Type::operator==(other) && (other.angle == angle)
			&& (other.runtimeDifference == runtimeDifference)
			&& (other.angleVariance == angleVariance)
			&& (other.runtimeDifferenceVariance == runtimeDifferenceVariance);

}

std::ostream& operator<<(std::ostream& o, Voice const& voice) {
	return o << "Voice [angle = " << voice.getAngle(iAU) << ", angleVariance = "
			<< voice.getAngleVariance(iAU) << ", runtimeDifferenceMicro = "
			<< voice.getRuntimeDifference(iTU)
			<< ", runtimeDifferenceVarianceMicro = "
			<< voice.getRuntimeDifferenceVariance(iTU) << "]";
}

void Voice::sanitizeLocation(xmltio::Location &loc) const {

}

void Voice::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@angle")] = getAngle(iAU);
	loc[XPath("POSITION/@angleVariance")] = getAngleVariance(iAU);

	loc[XPath("POSITION/@runtimeDifferenceMicro")] = getRuntimeDifference(iTU);
	loc[XPath("POSITION/@runtimeDifferenceVarianceMicro")] =
			getRuntimeDifferenceVariance(iTU);

}

void Voice::fromLocation(const xmltio::Location &loc_, Voice &voice) {

	Type::fromLocation(loc_, voice);

	voice.setAngle(extract<float>(loc_[XPath("POSITION/@angle")]), iAU);
	voice.setAngleVariance(
			extract<float>(loc_[XPath("POSITION/@angleVariance")]), iAU);

	voice.setRuntimeDifference(
			extract<float>(loc_[XPath("POSITION/@runtimeDifferenceMicro")]), iTU);
	voice.setRuntimeDifferenceVariance(
			extract<float>(loc_[XPath("POSITION/@runtimeDifferenceVarianceMicro")]),
			iTU);

}
}
