/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.grasp;

import de.unibi.airobots.btl.Type;
import nu.xom.Attribute;
import nu.xom.Element;

public class KatanaGripperData extends Type {

	private double forceRightInsideNear;
	private double forceRightInsideFar;
	private double infraredRightOutside;
	private double infraredRightFront;
	private double infraredRightInsideNear;
	private double infraredRightInsideFar;

	private double forceLeftInsideNear;
	private double forceLeftInsideFar;
	private double infraredLeftOutside;
	private double infraredLeftFront;
	private double infraredLeftInsideNear;
	private double infraredLeftInsideFar;

	private double infraredMiddle;

	/**
	 * Constructor.
	 */
	public KatanaGripperData() {
		super();
	}

	public static String getBaseTag() {
		return "SENSORS";
	}

	public double getForceRightInsideNear() {
		return forceRightInsideNear;
	}

	public void setForceRightInsideNear(double forceRightInsideNear) {
		this.forceRightInsideNear = forceRightInsideNear;
	}

	public double getForceRightInsideFar() {
		return forceRightInsideFar;
	}

	public void setForceRightInsideFar(double forceRightInsideFar) {
		this.forceRightInsideFar = forceRightInsideFar;
	}

	public double getInfraredRightOutside() {
		return infraredRightOutside;
	}

	public void setInfraredRightOutside(double infraredRightOutside) {
		this.infraredRightOutside = infraredRightOutside;
	}

	public double getInfraredRightFront() {
		return infraredRightFront;
	}

	public void setInfraredRightFront(double infraredRightFront) {
		this.infraredRightFront = infraredRightFront;
	}

	public double getInfraredRightInsideNear() {
		return infraredRightInsideNear;
	}

	public void setInfraredRightInsideNear(double infraredRightInsideNear) {
		this.infraredRightInsideNear = infraredRightInsideNear;
	}

	public double getInfraredRightInsideFar() {
		return infraredRightInsideFar;
	}

	public void setInfraredRightInsideFar(double infraredRightInsideFar) {
		this.infraredRightInsideFar = infraredRightInsideFar;
	}

	public double getForceLeftInsideNear() {
		return forceLeftInsideNear;
	}

	public void setForceLeftInsideNear(double forceLeftInsideNear) {
		this.forceLeftInsideNear = forceLeftInsideNear;
	}

	public double getForceLeftInsideFar() {
		return forceLeftInsideFar;
	}

	public void setForceLeftInsideFar(double forceLeftInsideFar) {
		this.forceLeftInsideFar = forceLeftInsideFar;
	}

	public double getInfraredLeftOutside() {
		return infraredLeftOutside;
	}

	public void setInfraredLeftOutside(double infraredLeftOutside) {
		this.infraredLeftOutside = infraredLeftOutside;
	}

	public double getInfraredLeftFront() {
		return infraredLeftFront;
	}

	public void setInfraredLeftFront(double infraredLeftFront) {
		this.infraredLeftFront = infraredLeftFront;
	}

	public double getInfraredLeftInsideNear() {
		return infraredLeftInsideNear;
	}

	public void setInfraredLeftInsideNear(double infraredLeftInsideNear) {
		this.infraredLeftInsideNear = infraredLeftInsideNear;
	}

	public double getInfraredLeftInsideFar() {
		return infraredLeftInsideFar;
	}

	public void setInfraredLeftInsideFar(double infraredLeftInsideFar) {
		this.infraredLeftInsideFar = infraredLeftInsideFar;
	}

	public double getInfraredMiddle() {
		return infraredMiddle;
	}

	public void setInfraredMiddle(double infraredMiddle) {
		this.infraredMiddle = infraredMiddle;
	}

	public static void fromElement(Element element, KatanaGripperData type) {

		Type.fromElement(element, type);

		/*
		 * right finger
		 */
		type.setForceRightInsideNear(Double.parseDouble(element
				.query("FINGER[@position=\"right\"]/"
						+ "FORCE[@name=\"inside_near\"]/@value").get(0)
				.getValue()));
		type.setForceRightInsideFar(Double.parseDouble(element
				.query("FINGER[@position=\"right\"]/"
						+ "FORCE[@name=\"inside_far\"]/@value").get(0)
				.getValue()));
		type.setInfraredRightOutside(Double.parseDouble(element
				.query("FINGER[@position=\"right\"]/"
						+ "INFRARED[@name=\"outside\"]/@value").get(0)
				.getValue()));
		type.setInfraredRightFront(Double
				.parseDouble(element
						.query("FINGER[@position=\"right\"]/"
								+ "INFRARED[@name=\"front\"]/@value").get(0)
						.getValue()));
		type.setInfraredRightInsideNear(Double.parseDouble(element
				.query("FINGER[@position=\"right\"]/"
						+ "INFRARED[@name=\"inside_near\"]/@value").get(0)
				.getValue()));
		type.setInfraredRightInsideFar(Double.parseDouble(element
				.query("FINGER[@position=\"right\"]/"
						+ "INFRARED[@name=\"inside_far\"]/@value").get(0)
				.getValue()));

		/*
		 * left finger
		 */
		type.setForceLeftInsideNear(Double.parseDouble(element
				.query("FINGER[@position=\"left\"]/"
						+ "FORCE[@name=\"inside_near\"]/@value").get(0)
				.getValue()));
		type.setForceLeftInsideFar(Double.parseDouble(element
				.query("FINGER[@position=\"left\"]/"
						+ "FORCE[@name=\"inside_far\"]/@value").get(0)
				.getValue()));
		type.setInfraredLeftOutside(Double.parseDouble(element
				.query("FINGER[@position=\"left\"]/"
						+ "INFRARED[@name=\"outside\"]/@value").get(0)
				.getValue()));
		type.setInfraredLeftFront(Double
				.parseDouble(element
						.query("FINGER[@position=\"left\"]/"
								+ "INFRARED[@name=\"front\"]/@value").get(0)
						.getValue()));
		type.setInfraredLeftInsideNear(Double.parseDouble(element
				.query("FINGER[@position=\"left\"]/"
						+ "INFRARED[@name=\"inside_near\"]/@value").get(0)
				.getValue()));
		type.setInfraredLeftInsideFar(Double.parseDouble(element
				.query("FINGER[@position=\"left\"]/"
						+ "INFRARED[@name=\"inside_far\"]/@value").get(0)
				.getValue()));

		/*
		 * middle
		 */
		type.setInfraredMiddle(Double.parseDouble(element
				.query("MIDDLE/INFRARED[@name=\"middle\"]/@value").get(0)
				.getValue()));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		/*
		 * right finger
		 */
		Element rightFingerElement = new Element("FINGER");
		parent.appendChild(rightFingerElement);
		rightFingerElement.addAttribute(new Attribute("position", "right"));
		Element forceRightInsideNear = new Element("FORCE");
		rightFingerElement.appendChild(forceRightInsideNear);
		forceRightInsideNear.addAttribute(new Attribute("name", "inside_near"));
		forceRightInsideNear.addAttribute(new Attribute("value", String
				.valueOf(getForceRightInsideNear())));
		Element forceRightInsideFar = new Element("FORCE");
		rightFingerElement.appendChild(forceRightInsideFar);
		forceRightInsideFar.addAttribute(new Attribute("name", "inside_far"));
		forceRightInsideFar.addAttribute(new Attribute("value", String
				.valueOf(getForceRightInsideFar())));
		Element infraredRightOutside = new Element("INFRARED");
		rightFingerElement.appendChild(infraredRightOutside);
		infraredRightOutside.addAttribute(new Attribute("name", "outside"));
		infraredRightOutside.addAttribute(new Attribute("value", String
				.valueOf(getInfraredRightOutside())));
		Element infraredRightFront = new Element("INFRARED");
		rightFingerElement.appendChild(infraredRightFront);
		infraredRightFront.addAttribute(new Attribute("name", "front"));
		infraredRightFront.addAttribute(new Attribute("value", String
				.valueOf(getInfraredRightFront())));
		Element infraredRightInsideNear = new Element("INFRARED");
		rightFingerElement.appendChild(infraredRightInsideNear);
		infraredRightInsideNear.addAttribute(new Attribute("name",
				"inside_near"));
		infraredRightInsideNear.addAttribute(new Attribute("value", String
				.valueOf(getInfraredRightInsideNear())));
		Element infraredRightInsideFar = new Element("INFRARED");
		rightFingerElement.appendChild(infraredRightInsideFar);
		infraredRightInsideFar
				.addAttribute(new Attribute("name", "inside_far"));
		infraredRightInsideFar.addAttribute(new Attribute("value", String
				.valueOf(getInfraredRightInsideFar())));

		/*
		 * left finger
		 */
		Element leftFingerElement = new Element("FINGER");
		parent.appendChild(leftFingerElement);
		leftFingerElement.addAttribute(new Attribute("position", "left"));
		Element forceLeftInsideNear = new Element("FORCE");
		leftFingerElement.appendChild(forceLeftInsideNear);
		forceLeftInsideNear.addAttribute(new Attribute("name", "inside_near"));
		forceLeftInsideNear.addAttribute(new Attribute("value", String
				.valueOf(getForceLeftInsideNear())));
		Element forceLeftInsideFar = new Element("FORCE");
		leftFingerElement.appendChild(forceLeftInsideFar);
		forceLeftInsideFar.addAttribute(new Attribute("name", "inside_far"));
		forceLeftInsideFar.addAttribute(new Attribute("value", String
				.valueOf(getForceLeftInsideFar())));
		Element infraredLeftOutside = new Element("INFRARED");
		leftFingerElement.appendChild(infraredLeftOutside);
		infraredLeftOutside.addAttribute(new Attribute("name", "outside"));
		infraredLeftOutside.addAttribute(new Attribute("value", String
				.valueOf(getInfraredLeftOutside())));
		Element infraredLeftFront = new Element("INFRARED");
		leftFingerElement.appendChild(infraredLeftFront);
		infraredLeftFront.addAttribute(new Attribute("name", "front"));
		infraredLeftFront.addAttribute(new Attribute("value", String
				.valueOf(getInfraredLeftFront())));
		Element infraredLeftInsideNear = new Element("INFRARED");
		leftFingerElement.appendChild(infraredLeftInsideNear);
		infraredLeftInsideNear
				.addAttribute(new Attribute("name", "inside_near"));
		infraredLeftInsideNear.addAttribute(new Attribute("value", String
				.valueOf(getInfraredLeftInsideNear())));
		Element infraredLeftInsideFar = new Element("INFRARED");
		leftFingerElement.appendChild(infraredLeftInsideFar);
		infraredLeftInsideFar.addAttribute(new Attribute("name", "inside_far"));
		infraredLeftInsideFar.addAttribute(new Attribute("value", String
				.valueOf(getInfraredLeftInsideFar())));

		/*
		 * middle
		 */
		Element middleElement = new Element("MIDDLE");
		parent.appendChild(middleElement);
		Element middleInfrared = new Element("INFRARED");
		middleElement.appendChild(middleInfrared);
		middleInfrared.addAttribute(new Attribute("name", "middle"));
		middleInfrared.addAttribute(new Attribute("value", String
				.valueOf(getInfraredMiddle())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "KatanaGripperData [" +

		"Force right inside near: " + forceRightInsideNear
				+ "Force right inside far: " + forceRightInsideFar
				+ "Infrared right outside: " + infraredRightOutside
				+ "Infrared right front: " + infraredRightFront
				+ "Infrared right inside near: " + infraredRightInsideNear
				+ "Infrared right inside far: " + infraredRightInsideFar +

				"Force left inside near: " + forceLeftInsideNear
				+ "Force left inside far: " + forceLeftInsideFar
				+ "Infrared left outside: " + infraredLeftOutside
				+ "Infrared left front: " + infraredLeftFront
				+ "Infrared left inside near: " + infraredLeftInsideNear
				+ "Infrared left inside far: " + infraredLeftInsideFar +

				"Infrared middle: " + infraredMiddle + "]";
	}

}
