/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.taskpattern;

import nu.xom.Attribute;
import nu.xom.Element;

import org.apache.log4j.Logger;

/**
 * This class provides the task pattern status tag and its attributes which is
 * equal and compatible to the status tag and the xtt project status tag
 * required to create a task pattern. It also builds the status tag, fills the
 * attributes. <STATUS value='<TaskStateData>' origin='TaskOriginData'>.
 * 
 * @author arothert
 */
public class TaskPatternStatusTagData {

	public enum TaskPatternStatus {
		// status tag name
		STATUS,
		// atrribute name for the task origin (TaskOriginData)
		origin,
		// atrribute name for the task state (TaskStateData)
		value
	}

	private static Logger logger = Logger
			.getLogger(TaskPatternStatusTagData.class);
	private Element status;
	private TaskOriginData origin;
	private TaskStateData state;

	public TaskPatternStatusTagData() {

	}

	public TaskPatternStatusTagData(TaskOriginData taskOrigin,
			TaskStateData taskState) {
		origin = taskOrigin;
		state = taskState;
	}

	public void fillTaskStatus(TaskOriginData taskOrigin,
			TaskStateData taskState) {
		origin = taskOrigin;
		state = taskState;
		status = new Element(TaskPatternStatus.STATUS.toString());
		status.addAttribute(new Attribute(TaskPatternStatus.origin.toString(),
				taskOrigin.toString()));
		status.addAttribute(new Attribute(TaskPatternStatus.value.toString(),
				taskState.toString()));
	}

	public static Element createTaskStatusElement(TaskOriginData taskOrigin,
			TaskStateData taskState) {
		Element statusElement = new Element(TaskPatternStatus.STATUS.toString());
		statusElement.addAttribute(new Attribute(TaskPatternStatus.origin
				.toString(), taskOrigin.toString()));
		statusElement.addAttribute(new Attribute(TaskPatternStatus.value
				.toString(), taskState.toString()));
		return statusElement;
	}

	public Element getTaskStatus() {
		if (state != null && origin != null) {
			fillTaskStatus(origin, state);
			return status;
		} else {
			logger.debug("state and origin must be filled before!");
			return null;
		}
	}

	public void setOrigin(TaskOriginData origin) {
		this.origin = origin;
	}

	public TaskOriginData getOrigin() {
		return origin;
	}

	public void setState(TaskStateData state) {
		this.state = state;
	}

	public TaskStateData getState() {
		return state;
	}

}
