/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * CameraPositionDataTest.cpp
 *
 *  Created on: Apr 10, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/data/hardwareinfo/CameraPositionData.h"
#include "btl/units/AngleUnit.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(CameraPositionDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		CameraPositionData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setPan(-0.456,RADIAN);
		original.setTilt(12.45,RADIAN);
		original.setZoom(0.2345);

		Location originalLoc(CameraPositionData::getTemplate(), CameraPositionData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		CameraPositionData parsed = extract<CameraPositionData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getPan(RADIAN), original.getPan(RADIAN), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTilt(RADIAN), original.getTilt(RADIAN), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getZoom(), original.getZoom(), TOLERANCE);

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("CameraPosition.xml"), CameraPositionData::getBaseTag());
	CameraPositionData parsed = extract<CameraPositionData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(123.0, parsed.getTilt(RADIAN), 0.0001);
	BOOST_REQUIRE_CLOSE(0.345, parsed.getPan(RADIAN), 0.0001);
	BOOST_REQUIRE_CLOSE(0.75, parsed.getZoom(), 0.75);

}

BOOST_AUTO_TEST_SUITE_END()
