/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>
#include <math.h>

#include "btl/units/UnitConverter.h"

using namespace airobots;
using namespace std;

BOOST_AUTO_TEST_SUITE(BinarySlamMapTest)

BOOST_AUTO_TEST_CASE(testConvert) {

	const float DELTA = 0.001f;

	// ############# LENGTH ###############

	// INTEGER
	BOOST_REQUIRE_EQUAL(1, UnitConverter::convert(1, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(0, UnitConverter::convert(1, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(0, UnitConverter::convert(1, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100, UnitConverter::convert(1000, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1, UnitConverter::convert(1000, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(10000, UnitConverter::convert(1000, CENTIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, CENTIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(10, UnitConverter::convert(1000, CENTIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000000, UnitConverter::convert(1000, METER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100000, UnitConverter::convert(1000, METER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, METER, METER));

	// LONG
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100l, UnitConverter::convert(1000l, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(10000l, UnitConverter::convert(1000l, CENTIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, CENTIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(10l, UnitConverter::convert(1000l, CENTIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000000l, UnitConverter::convert(1000l, METER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100000l, UnitConverter::convert(1000l, METER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, METER, METER));

	// FLOAT
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.1f, UnitConverter::convert(1.0f, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.001f, UnitConverter::convert(1.0f, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100.0f, UnitConverter::convert(1000.0f, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1000.0f, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(10000.0f, UnitConverter::convert(1000.0f, CENTIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, CENTIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(10.0f, UnitConverter::convert(1000.0f, CENTIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000000.0f, UnitConverter::convert(1000.0f, METER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100000.0f, UnitConverter::convert(1000.0f, METER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, METER, METER), DELTA);

	// DOUBLE
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.1, UnitConverter::convert(1.0, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.001, UnitConverter::convert(1.0, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100.0, UnitConverter::convert(1000.0, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1000.0, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(10000.0, UnitConverter::convert(1000.0, CENTIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, CENTIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(10.0, UnitConverter::convert(1000.0, CENTIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000000.0, UnitConverter::convert(1000.0, METER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100000.0, UnitConverter::convert(1000.0, METER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, METER, METER), DELTA);


	// ############# ANGLE ###############

	// FLOAT
	BOOST_REQUIRE_CLOSE((float) M_PI, UnitConverter::convert(180.0f, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE((float) M_PI/2.0f, UnitConverter::convert(90.0f, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE((float) -M_PI/2.0f, UnitConverter::convert(-90.0f, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, DEGREE, RADIAN), DELTA);

	BOOST_REQUIRE_CLOSE(180.0f, UnitConverter::convert((float) M_PI, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(90.0f, UnitConverter::convert((float) M_PI/2.0f, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(-90.0f, UnitConverter::convert((float) -M_PI/2.0f, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, RADIAN, DEGREE), DELTA);

	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, RADIAN, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, RADIAN, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, DEGREE, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, DEGREE, DEGREE), DELTA);

	// DOUBLE
	BOOST_REQUIRE_CLOSE(M_PI, UnitConverter::convert(180.0, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(M_PI/2.0, UnitConverter::convert(90.0, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(-M_PI/2.0, UnitConverter::convert(-90.0, DEGREE, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, DEGREE, RADIAN), DELTA);

	BOOST_REQUIRE_CLOSE(180.0, UnitConverter::convert(M_PI, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(90.0, UnitConverter::convert(M_PI/2.0, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(-90.0, UnitConverter::convert(-M_PI/2.0, RADIAN, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, RADIAN, DEGREE), DELTA);

	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, RADIAN, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, RADIAN, RADIAN), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, DEGREE, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, DEGREE, DEGREE), DELTA);

	// ############# SPEED ###############

	// FLOAT
	BOOST_REQUIRE_CLOSE((float) 1.0/3.6, UnitConverter::convert(1.0f, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE((float) -1.0/3.6, UnitConverter::convert(-1.0f, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);

	BOOST_REQUIRE_CLOSE(3.6f, UnitConverter::convert(1.f, METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(-3.6f, UnitConverter::convert(-1.f, METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);

	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, METER_PER_SEC, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, METER_PER_SEC, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, KILOMETER_PER_HOUR, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, KILOMETER_PER_HOUR, KILOMETER_PER_HOUR), DELTA);

	// DOUBLE
	BOOST_REQUIRE_CLOSE(1.0/3.6, UnitConverter::convert(1.0, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(-1.0/3.6, UnitConverter::convert(-1.0, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, KILOMETER_PER_HOUR, METER_PER_SEC), DELTA);

	BOOST_REQUIRE_CLOSE(3.6, UnitConverter::convert(1., METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(-3.6, UnitConverter::convert(-1., METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, METER_PER_SEC, KILOMETER_PER_HOUR), DELTA);

	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, METER_PER_SEC, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, METER_PER_SEC, METER_PER_SEC), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, KILOMETER_PER_HOUR, KILOMETER_PER_HOUR), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, KILOMETER_PER_HOUR, KILOMETER_PER_HOUR), DELTA);

	// ############# TIME ###############

	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, MICROSECONDS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1l, MILLISECONDS, MICROSECONDS));
	/*
	BOOST_REQUIRE_EQUAL(1000l * 1000l, UnitConverter::convert(1l, SECONDS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l, UnitConverter::convert(1l, MINUTES, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l * 60l, UnitConverter::convert(1l, HOURS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l * 60l * 24l, UnitConverter::convert(1l, DAYS, MICROSECONDS));

	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MICROSECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLISECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, SECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MINUTES, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, HOURS, DAYS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, DAYS, DAYS));

	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l * 1000L, MICROSECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l, MILLISECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, SECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(60l, UnitConverter::convert(1l, MINUTES, SECONDS));
	BOOST_REQUIRE_EQUAL(60l * 60l, UnitConverter::convert(1l, HOURS, SECONDS));
	BOOST_REQUIRE_EQUAL(60l * 60l * 24l, UnitConverter::convert(1l, DAYS, SECONDS));
*/
}

BOOST_AUTO_TEST_SUITE_END()
