/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.speechrec;

import static org.junit.Assert.assertEquals;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;

/**
 * Test cases for {@link Type}.
 * 
 * @author hterhors
 */
public class GPSRActionListTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		GPSRActionList original = new GPSRActionList();
		// final long timestamp = System.currentTimeMillis();
		GPSRAction action1 = new GPSRAction();
		action1.setName("goTo");
		action1.addParam(action1.new GPSRParameter("kitchen", "location"));
		GPSRAction action2 = new GPSRAction();
		action2.setName("getObject");
		action2.addParam(action2.new GPSRParameter("cup", "object"));
		GPSRAction action3 = new GPSRAction();
		action3.setName("putObject");
		action3.addParam(action3.new GPSRParameter("cup", "object"));
		action3.addParam(action3.new GPSRParameter("table", "loction"));

		original.addAction(action1);
		original.addAction(action2);
		original.addAction(action3);

		Document doc = new Document(new Element(GPSRActionList.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		GPSRActionList parsed = new GPSRActionList();
		GPSRActionList.fromElement(doc.getRootElement(), parsed);

		// assertEquals(original.getGenerator(), parsed.getGenerator());
		// assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getActions().get(0), parsed.getActions().get(0));
		assertEquals(original.getActions().get(1), parsed.getActions().get(1));
		assertEquals(original.getActions().get(2), parsed.getActions().get(2));

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("GPSRActionListTest.xml"));

		GPSRActionList parsed = new GPSRActionList();
		GPSRActionList.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				GPSRActionList.getBaseTag());

		// GPSRActionList original = new GPSRActionList();
		// final long timestamp = System.currentTimeMillis();
		// original.setTimestamp(timestamp);
		// original.setGenerator("test");
		GPSRAction action1 = new GPSRAction();
		action1.setName("goTo");
		action1.addParam(action1.new GPSRParameter("kitchen", "location"));
		GPSRAction action2 = new GPSRAction();
		action2.setName("getObject");
		action2.addParam(action2.new GPSRParameter("cup", "object"));
		GPSRAction action3 = new GPSRAction();
		action3.setName("putObject");
		action3.addParam(action3.new GPSRParameter("cup", "object"));
		action3.addParam(action3.new GPSRParameter("table", "location"));

		// assertEquals("test", parsed.getGenerator());
		// assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(action1, parsed.getActions().get(0));
		assertEquals(action2, parsed.getActions().get(1));
		assertEquals(action3, parsed.getActions().get(2));

	}

}
