/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.vision3d;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.Timestamp;
import de.unibi.airobots.btl.data.geometry.Point3D;
import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.data.geometry.Rotation3D;
import de.unibi.airobots.btl.data.vision2d.RegionData.Scope;
import de.unibi.airobots.btl.data.vision3d.PlaneData;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class PlaneDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		PlaneData original = new PlaneData();
		original.setScope(Scope.LOCAL);
		Point3D origin = new Point3D(1.0, 1.0, 1.0, LengthUnit.MILLIMETER);
		original.setOrigin(origin);
		Rotation3D rotation = new Rotation3D();
		original.setRotation(rotation);

		Document doc = new Document(new Element(PlaneData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PlaneData parsed = new PlaneData();
		PlaneData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getScope(), parsed.getScope());
		assertEquals(original.getPolygon(), parsed.getPolygon());

		assertEquals(original.getOrigin().getX(LengthUnit.MILLIMETER), parsed
				.getOrigin().getX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getOrigin().getY(LengthUnit.MILLIMETER), parsed
				.getOrigin().getY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(original.getOrigin().getZ(LengthUnit.MILLIMETER), parsed
				.getOrigin().getZ(LengthUnit.MILLIMETER), 0.0001);

		assertEquals(original.getRotation(), parsed.getRotation());

		System.out.println(parsed);
	}

	@Test
	public void fileCompatibility() throws Exception {

		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(0d, 0d, LengthUnit.MILLIMETER);
		poly.addPoint(1d, 1d, LengthUnit.MILLIMETER);
		poly.addPoint(2d, 2d, LengthUnit.MILLIMETER);

		Builder parser = new Builder();
		Document doc = parser
				.build(TestUtils.makeTestFileName("PlaneData.xml"));

		PlaneData parsed = new PlaneData();
		PlaneData.fromElement(doc.getRootElement(), parsed);

		assertEquals(doc.getRootElement().getLocalName(),
				PlaneData.getBaseTag());

		assertEquals(new Timestamp(253543797l, 253543712l),
				parsed.getTimestamp());
		assertEquals(Scope.LOCAL, parsed.getScope());
		assertEquals(poly, parsed.getPolygon());

		assertEquals(1.0, parsed.getOrigin().getX(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(1.0, parsed.getOrigin().getY(LengthUnit.MILLIMETER), 0.0001);
		assertEquals(1.0, parsed.getOrigin().getZ(LengthUnit.MILLIMETER), 0.0001);

		assertEquals(new Rotation3D(), parsed.getRotation());
	}
}
